package edu.dhbw.concurrent;

public class Matrix {

    protected double[][] val;

    public Matrix(double[][] v) {
        val = v;
    }

    public Matrix(int n, int m) {
        this(new double[n][m]);
    }

    public String toString() {
        StringBuffer sb = new StringBuffer("\n");
        for (int i = 0; i < val.length; i++) {
            for (int j = 0; j < val[0].length; j++) {
                sb.append(val[i][j]);
                sb.append(" ");
            }
            sb.append("\n");
        }
        return sb.toString();
    }

    public static Matrix one(int n) {
        double[][] v = new double[n][n];
        for (int i = 0; i < n; i++) {
            v[i][i] = 1.0;
        }
        return new Matrix(v);
    }

    public static Matrix random(int n, int m) {
        double[][] v = new double[n][m];
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                v[i][j] = Math.floor(Math.random() * 10);
            }
        }
        return new Matrix(v);
    }

    public Matrix diff(Matrix B) {
        double[][] v = new double[val.length][val[0].length];
        double[][] Bval = B.val;
        for (int i = 0; i < val.length; i++) {
            for (int j = 0; j < val[0].length; j++) {
                v[i][j] = val[i][j] - Bval[i][j];
            }
        }
        return new Matrix(v);
    }

    public Matrix transpose() {
        double[][] v = new double[val[0].length][val.length];
        for (int i = 0; i < val.length; i++) {
            for (int j = 0; j < val[0].length; j++) {
                v[j][i] = val[i][j];
            }
        }
        return new Matrix(v);
    }

    static double epsilon = 1e-10;

    public boolean isZero() {
        for (int i = 0; i < val.length; i++) {
            for (int j = 0; j < val[0].length; j++) {
                if (Math.abs(val[i][j]) >= epsilon) {
                    return false;
                }
            }
        }
        return true;
    }

    public Matrix mult(Matrix B) {
        double[][] Bval = B.val;
        double[][] c = new double[val.length][Bval[0].length];
        for (int i = 0; i < val.length; i++) {
            for (int j = 0; j < Bval[0].length; j++) {
                double s = 0.0;
                for (int k = 0; k < val[0].length; k++) {
                    s += val[i][k] * Bval[k][j];
                }
                c[i][j] = s;
            }
        }
        return new Matrix(c);
    }
}
