using System;
using System.Threading;
using System.Threading.Tasks;

// from: https://docs.microsoft.com/de-de/dotnet/standard/parallel-programming/

class CustomData
{
   public long CreationTime;
   public long RunTime;
   public int Name; 
   public int ThreadNum;
}

public class TaskExample
{
   public static void Main(string[] args)
   {
      long StartTime = DateTime.Now.Ticks;
      Console.WriteLine("StartTime {0}.", StartTime);
      Task[] taskArray = new Task[10];
      for (int i = 0; i < taskArray.Length; i++) {
         taskArray[i] = Task.Factory.StartNew( (Object obj ) => {
                                                  CustomData data = obj as CustomData;
                                                  if (data == null) 
                                                     return;
                                                  data.ThreadNum = Thread.CurrentThread.ManagedThreadId;
                                                  data.RunTime = DateTime.Now.Ticks - data.CreationTime;
                                               },
                                               new CustomData() {Name = i, CreationTime = DateTime.Now.Ticks} );
      }
      Task.WaitAll(taskArray);     
      foreach (var task in taskArray) {
         var data = task.AsyncState as CustomData;
         if (data != null)
            Console.WriteLine("Task #{0} created at {1}, ran on thread #{2} at {3} ticks after creation.",
                              data.Name, data.CreationTime-StartTime, data.ThreadNum, data.RunTime);
      }                     
   }
}
