package sumthatsine

import (
	"fmt"
	"math"

	"../measuretime"
)

const N = 1e8

func SineMain() {
	fmt.Printf("Summarize sine with %v steps.\n", N)

	var timer measuretime.Timer
	timer.Pre()
	seqResult := sumThatSineSeq(N)
	timer.Post()
	fmt.Printf("Result: %.5f\n", seqResult)
	timer.PrintTimeConsumption("sequential algorithm")

	timer.Pre()
	parResult := sumThatSinePar(N)
	timer.Post()
	fmt.Printf("Result: %.5f\n", parResult)
	timer.PrintTimeConsumption("parallel algorithm")
}

func thatSine(x float64) float64 {
	return math.Sin(2 * x)
}

func sumThatSineSeq(n int) float64 {
	var sum float64
	for i := 0; i < n; i++ {
		sum += thatSine(float64(i))
	}
	return sum
}

func sumThatSinePar(n int) float64 {
	const nThreads = 4
	sumChannel := make(chan float64, 5)
	var lower, upper int
	for i := 0; i < nThreads; i++ {
		lower = i * n / nThreads
		upper = (i + 1) * n / nThreads
		go sumThatSineWorker(lower, upper, sumChannel)
	}

	var sum float64
	for i := 0; i < nThreads; i++ {
		sum += <-sumChannel
	}
	close(sumChannel)
	return sum
}

func sumThatSineWorker(lower, upper int, sumChan chan float64) {
	var sum float64
	for i := lower; i < upper; i++ {
		sum += thatSine(float64(i))
	}
	sumChan <- sum
}
