//package edu.unima.ky.parallel;

/** 
 * Buchung.
 * Multi threaded finance testing application for the "Bounded Buffer"
 * generates random transfer amount within desired range.
 * @author Heinz Kredel. 
 */ 
public class Buchung {

/**
 * Main method.
 * @param args
 * @throws InterruptedException
 */
    public static void main(String[] args) throws InterruptedException {

        int anzahl = 100; // number of iterations
        int kanzahl = 10; // number of accounts
        BoundedBuffer buffer = new BoundedBuffer(10); // Buffers are generated here.
        KontoA[] konten = new KontoA[kanzahl];  
        double balance = 0.0;
        double stand;
        for (int i = 0; i < konten.length; i++ ) {
            konten[i] = new KontoA( 0.0 );
            stand = konten[i].zeigeGeld();
            balance += stand;
            System.out.println("Konto " + i 
                            + " Stand " + stand);
        }
        System.out.println("Balance " + balance); 
        

        Thread t1 = new BuchungInput(anzahl,buffer,konten);
/*
 * Thread t2 gets forth size of iterations.
 * Thread t3 and thread t4 gets each a half.
 */     
        Thread t2 = new BuchungDo(anzahl/2,buffer,"1");
        Thread t3 = new BuchungDo(anzahl/4,buffer,"2");
        Thread t4 = new BuchungDo(anzahl/4,buffer,"3");
     
        t1.start(); t2.start(); t3.start(); t4.start();
        t1.join(); t2.join(); t3.join(); t4.join();

        balance = 0.0;
        for (int i = 0; i < konten.length; i++ ) {
            stand = konten[i].zeigeGeld();
            balance += stand;
            System.out.println("Konto " + i 
                            + " Stand " + stand);
        }
        System.out.println("Balance " + balance); 

    }

}

/** 
 * Generates input data for Buchung.
 * Uses an array KontoA[] "konten", int "anzahl" as 
 * number of accounts, "kmax" as konten.length and bmax as range 
 * of amount to transfer.
 */
class BuchungInput extends Thread {
    private KontoA[] konten; 
    private int anzahl, kmax;
    private double bmax;
    private BoundedBuffer buffer;

/**
 * Generates accouting data.
 * @param a size of the buffer.
 * @param b BoundedBuffer.
 * @param k KontoA. 
 */
    BuchungInput(int a, BoundedBuffer b, KontoA[] k) {
        anzahl = a;
        buffer = b;
        konten = k;
        kmax = konten.length;
        bmax = 100000.0;
    }

/**
 * Run in parallel.
 */
    public void run() {
        for (int i = 0; i < anzahl; i++) {
            int a = (int) ( Math.random() * kmax ); 
            int b = (int) ( Math.random() * kmax ); 
            double betrag = (double) ( Math.random() * bmax ); 
            Ueberweisung u = new Ueberweisung( konten[a], konten[b], betrag );
            System.out.print("+");
            //try {
                 buffer.put( (Object)u );
            //} catch (InterruptedException e) { }
        }
        System.out.println();
    }

}

/**
 * Process the Buchung data.
 * Uses a buffer ("BoundedBuffer"), Int anzahl and String me.
 */
class BuchungDo extends Thread {
    private int anzahl;
    private BoundedBuffer buffer;
    private String me = "-";

/**
 * This constructor generates int "anzahl" (number of accounts),
 * a BoundedBuffer and String "m".
 * @param a number of transactions.
 * @param b BoundedBuffer.
 * @param m String.
 */
    BuchungDo(int a, BoundedBuffer b, String m) {
        me = m;
        anzahl = a;
        buffer = b;
    }

    public void run() {
        for (int i = 0; i < anzahl; i++) {
            //try {
                Ueberweisung u = (Ueberweisung) buffer.get();
                System.out.print(me);
                u.run();
            //} catch (InterruptedException e) { }
        }
        System.out.println();
    }

}
