public class Matrix {

    static final double EPS = 1.0e-10;
    double[][] A;

    public Matrix(double[][] A) {
        this.A = A;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        for(int i = 0; i < A.length; i++) {
            String s = "";
            for (int j = 0; j < A[i].length; j++) {
                s += A[i][j] + " ";
            }
            sb.append(s + "\n");
        }
        return sb.toString();
    }

    public static Matrix generateOne(int n, int m) {
        double[][] one = new double[n][m];
        for(int i = 0; i < n; i++) {
            for ( int j = 0; j < m; j++) {
                one[i][j] = (i == j ? 1.0 : 0.0); 
            }
        }
        return new Matrix(one);
    }

    public static Matrix generateZero(int n, int m) {
        double[][] zero = new double[n][m];
        for(int i = 0; i < n; i++) {
            for ( int j = 0; j < m; j++) {
                zero[i][j] = 0.0; 
            }
        }
        return new Matrix(zero);
    }

    public static Matrix generateRandom(int n, int m) {
        double[][] ran = new double[n][m];
        for(int i = 0; i < n; i++) {
            for ( int j = 0; j < m; j++) {
                ran[i][j] = Math.random(); 
            }
        }
        return new Matrix(ran);
    }

    public boolean isNearlyZero() {
        for(int i = 0; i < A.length; i++) {
            for ( int j = 0; j < A[i].length; j++) {
                if ( Math.abs(A[i][j]) > EPS ) {
                    return false;
                }
            }
        }
        return true;
    }

    public Matrix difference(Matrix B) {
        double[][] diff = new double[A.length][A[0].length];
        double[][] mat = B.A;
        for(int i = 0; i < A.length; i++) {
            for (int j = 0; j < A[i].length; j++) {
                diff[i][j] = A[i][j] - mat[i][j];
            }
        }
        return new Matrix(diff);
    }
    
    public Matrix transpose() {
        double[][] tran = new double[A.length][A[0].length];
        for (int i = 0; i < A.length; i++) {
            for ( int j = 0; j < A[0].length; j++) {
                tran[i][j] = A[j][i];
            }
        }
        return new Matrix(tran);
    }
    
    public Matrix multiplyTrans(Matrix B) {
        Matrix Bt = B.transpose();
        double[][] mat = Bt.A;
        double[][] mul = new double[A.length][mat[0].length];
        for (int i = 0; i < A.length; i++ ) {
            for (int j = 0; j < mat[0].length; j++) {
                double c = 0.0;
                for (int k = 0; k < mat.length; k ++) {
                    c += A[i][k] * mat[j][k];
                }
                mul[i][j] = c;
            }
        }
        return new Matrix(mul);
    }

    public Matrix multiply(Matrix B) {
        double[][] mat = B.A;
        double[][] mul = new double[A.length][mat[0].length];
        for(int i = 0; i < A.length; i++) {
            for(int j = 0; j < mat[i].length; j++) {
                double s = 0.0;
                for (int k = 0; k < mat.length; k++) {
                    s += A[i][k] * mat[k][j];
                }
                mul[i][j] = s;
            }
                         
        }
        return new Matrix(mul);
    }
}
