
import java.util.ArrayList;

/**
 * A Path in a Graph implemented with ArrayLists.
 */

public final class PathArrayList implements Path {

    protected final Graph graph;
    protected final ArrayList used;   // points in the path
    protected final ArrayList unused; // points not in the path
    public final double cost;
    public final int maxsize;
    public final int length;

    public PathArrayList(Graph g, ArrayList u, ArrayList uu, double c) {
        this(g, u, u.size(), uu, c);
    }

    public PathArrayList(Graph g, ArrayList u, int pos, ArrayList uu, double c) {
        graph = g;
        used = u;
        unused = uu;
        cost = c;
        length = pos;
        maxsize = graph.size();
    }


    public double cost() {
        return cost;
    }

    public int length() {
        return length;
    }

    public int maxsize() {
        return maxsize;
    }


    /**
     * empty path, all n points unused.
     */

    public static PathArrayList emptyPath(Graph g) { 
        ArrayList u  = new ArrayList( g.size() );
        ArrayList uu = new ArrayList( g.size() );
        for (int i = 0; i < g.size(); i++ ){
            uu.add( i, new Integer(i) );
        }
        return new PathArrayList(g, u, uu, Double.POSITIVE_INFINITY);
    }


    /**
     * standard path from point 0 to point n-1 to 0.
     */

    public static PathArrayList standardPath(Graph g) {
        ArrayList u  = new ArrayList( g.size() );
        ArrayList uu = new ArrayList();
        double c = 0.0;
        u.add( 0, new Integer(0) );
        for (int i = 1; i < g.size(); i++ ){
            u.add( i, new Integer(i) );
            c += cost( g.distance(i-1,i) );
        }
        // u.add( g.size(), new Integer(0) );
        c += cost( g.distance(g.size()-1,0) ); //for circle
        PathArrayList p = new PathArrayList(g, u, uu, c);
        System.out.println("initial  = " + p);
        return p;
    }


    /**
     * return a copy of this with maximal used array.
     */

    public Path copyMax() {
        ArrayList u = (ArrayList)used.clone(); 
        return new PathArrayList( graph, u, length, unused, cost );
    }

    /**
     * get next possible path reusing used array.
     */

    public Path nextPath(int pos) {
	
        System.out.println(" pos = " + pos + " this = " + this);
	
        ArrayList uu = (ArrayList)unused.clone();
        Object j = uu.remove(pos);
        // j != null
        int k = ((Integer)j).intValue();
        ArrayList u = (ArrayList)used.clone();
        u.add( j );
        // j != null
        // compute new cost
        double c = 0.0;
        if ( cost < Double.POSITIVE_INFINITY ) {
           c = cost;
        }
        if ( length > 0 ) {
           int n1 = ((Integer)used.get( length-1 )).intValue();
           c += cost( graph.distance(n1,k) );
        }
        // add circle cost
        if ( length+1 == maxsize ) {
           //System.out.print("#");
           int n0 = ((Integer)used.get( 0 )).intValue();
           c += cost( graph.distance(k,n0) );
        }
        // new path shares used list, so do not use in threads
        return new PathArrayList( graph, used, length+1, uu, c ); 
    }


    /**
     * get array of next possible paths.
     */

    public Path[] nextPaths() {
        PathArrayList[] pr = new PathArrayList[ unused.size() ];
        for ( int i = 0; i < unused.size(); i++ ) {
            ArrayList uu = (ArrayList)unused.clone();
            Object j = uu.remove(i);
            // j != null
            int k = ((Integer)j).intValue();
            ArrayList u = (ArrayList)used.clone();
            u.add( j );
            // compute new cost
            double c = 0.0;
            if ( cost < Double.POSITIVE_INFINITY ) {
               c = cost;
            }
            if ( length > 0 ) {
               int n1 = ((Integer)used.get( length-1 )).intValue();
               c += cost( graph.distance(n1,k) );
            }
            // add circle cost
            if ( length+1 == maxsize ) {
                //System.out.print("#");
               int n0 = ((Integer)used.get( 0 )).intValue();
               c += cost( graph.distance(k,n0) );
            }
            pr[i] = new PathArrayList( graph, u, uu, c ); 
        }
        return pr;
    }


    public String toString() {
        StringBuffer s = new StringBuffer();
        s.append("[ ");
        for ( int i = 0; i < length; i++ ) {
            s.append( used.get(i) );
            if ( i+1 < length ) {
               s.append(",");
            }
        }
        s.append(" | cost="+cost+" ]");
        return s.toString();
    }

    protected static double cost(double c) {
        if ( c >= 0.0 ) {
            return c;
        }
        double r = Math.log( c );
        for (int i = 0; i < 100; i++ ) {
            r *= c;
        }
        r = r/2.0;
        for (int i = 0; i < 100; i++ ) {
            r /= c;
        }
        r = r/2.0;
        return Math.exp( r );
    }

}